@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2015 LOVE Development Team
+ * Copyright (c) 2006-2016 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -24,39 +24,23 @@
 namespace love
 {
 
-extern StringMap<Type, TYPE_MAX_ENUM> types;
-
 static love::Type extractudatatype(lua_State *L, int idx)
 {
-	Type t = INVALID_ID;
-	if (!lua_isuserdata(L, idx))
-		return t;
-	if (luaL_getmetafield(L, idx, "__tostring") == 0)
-		return t;
-	lua_pushvalue(L, idx);
-	int result = lua_pcall(L, 1, 1, 0);
-	if (result == 0)
-		types.find(lua_tostring(L, -1), t);
-	if (result == 0 || result == LUA_ERRRUN)
-		lua_pop(L, 1);
-	return t;
-}
+	Proxy *u = (Proxy *)lua_touserdata(L, idx);
 
-static inline void delete_table(std::vector<std::pair<Variant*, Variant*> > *table)
-{
-	while (!table->empty())
-	{
-		std::pair<Variant*, Variant*> &kv = table->back();
-		kv.first->release();
-		kv.second->release();
-		table->pop_back();
-	}
-	delete table;
+	if (u == nullptr || u->type <= INVALID_ID || u->type >= TYPE_MAX_ENUM)
+		return INVALID_ID;
+
+	// We could get rid of the dynamic_cast for more performance, but it would
+	// be less safe...
+	if (dynamic_cast<Object *>(u->object) != nullptr)
+		return u->type;
+
+	return INVALID_ID;
 }
 
 Variant::Variant()
 	: type(NIL)
-	, data()
 {
 }
 
@@ -73,19 +57,18 @@ Variant::Variant(double number)
 }
 
 Variant::Variant(const char *string, size_t len)
-	: type(STRING)
 {
-	char *buf = new char[len+1];
-	memset(buf, 0, len+1);
-	memcpy(buf, string, len);
-	data.string.str = buf;
-	data.string.len = len;
-}
-
-Variant::Variant(char c)
-	: type(CHARACTER)
-{
-	data.character = c;
+	if (len <= MAX_SMALL_STRING_LENGTH)
+	{
+		type = SMALLSTRING;
+		memcpy(data.smallstring.str, string, len);
+		data.smallstring.len = (uint8) len;
+	}
+	else
+	{
+		type = STRING;
+		data.string = new SharedString(string, len);
+	}
 }
 
 Variant::Variant(void *userdata)
@@ -101,19 +84,39 @@ Variant::Variant(love::Type udatatype, void *userdata)
 	if (udatatype != INVALID_ID)
 	{
 		Proxy *p = (Proxy *) userdata;
-		flags = p->flags;
-		data.userdata = p->data;
-		((love::Object *) data.userdata)->retain();
+		data.userdata = p->object;
+		p->object->retain();
 	}
 	else
 		data.userdata = userdata;
 }
 
 // Variant gets ownership of the vector.
-Variant::Variant(std::vector<std::pair<Variant*, Variant*> > *table)
+Variant::Variant(std::vector<std::pair<Variant, Variant>> *table)
 	: type(TABLE)
 {
-	data.table = table;
+	data.table = new SharedTable(table);
+}
+
+Variant::Variant(const Variant &v)
+	: type(v.type)
+	, udatatype(v.udatatype)
+	, data(v.data)
+{
+	if (type == STRING)
+		data.string->retain();
+	else if (type == FUSERDATA)
+		((love::Object *) data.userdata)->retain();
+	else if (type == TABLE)
+		data.table->retain();
+}
+
+Variant::Variant(Variant &&v)
+	: type(std::move(v.type))
+	, udatatype(std::move(v.udatatype))
+	, data(std::move(v.data))
+{
+	v.type = NIL;
 }
 
 Variant::~Variant()
@@ -121,113 +124,125 @@ Variant::~Variant()
 	switch (type)
 	{
 	case STRING:
-		delete[] data.string.str;
+		data.string->release();
 		break;
 	case FUSERDATA:
 		((love::Object *) data.userdata)->release();
 		break;
 	case TABLE:
-		delete_table(data.table);
+		data.table->release();
+		break;
 	default:
 		break;
 	}
 }
 
-Variant *Variant::fromLua(lua_State *L, int n, bool allowTables)
+Variant &Variant::operator = (const Variant &v)
+{
+	if (v.type == STRING)
+		v.data.string->retain();
+	else if (v.type == FUSERDATA)
+		((love::Object *) v.data.userdata)->retain();
+	else if (v.type == TABLE)
+		v.data.table->retain();
+
+	if (type == STRING)
+		data.string->release();
+	else if (type == FUSERDATA)
+		((love::Object *) v.data.userdata)->release();
+	else if (type == TABLE)
+		data.table->release();
+
+	type = v.type;
+	data = v.data;
+	udatatype = v.udatatype;
+
+	return *this;
+}
+
+Variant Variant::fromLua(lua_State *L, int n, bool allowTables)
 {
-	Variant *v = NULL;
 	size_t len;
 	const char *str;
+
 	if (n < 0) // Fix the stack position, we might modify it later
 		n += lua_gettop(L) + 1;
 
 	switch (lua_type(L, n))
 	{
 	case LUA_TBOOLEAN:
-		v = new Variant(luax_toboolean(L, n));
-		break;
+		return Variant(luax_toboolean(L, n));
 	case LUA_TNUMBER:
-		v = new Variant(lua_tonumber(L, n));
-		break;
+		return Variant(lua_tonumber(L, n));
 	case LUA_TSTRING:
 		str = lua_tolstring(L, n, &len);
-		v = new Variant(str, len);
-		break;
+		return Variant(str, len);
 	case LUA_TLIGHTUSERDATA:
-		v = new Variant(lua_touserdata(L, n));
-		break;
+		return Variant(lua_touserdata(L, n));
 	case LUA_TUSERDATA:
-		v = new Variant(extractudatatype(L, n), lua_touserdata(L, n));
-		break;
+		return Variant(extractudatatype(L, n), lua_touserdata(L, n));
 	case LUA_TNIL:
-		v = new Variant();
-		break;
+		return Variant();
 	case LUA_TTABLE:
 		if (allowTables)
 		{
 			bool success = true;
-			std::vector<std::pair<Variant*, Variant*> > *table = new std::vector<std::pair<Variant*, Variant*> >();
+			std::vector<std::pair<Variant, Variant>> *table = new std::vector<std::pair<Variant, Variant>>();
+
+			size_t len = luax_objlen(L, -1);
+			if (len > 0)
+				table->reserve(len);
+
 			lua_pushnil(L);
+
 			while (lua_next(L, n))
 			{
-				Variant *key = fromLua(L, -2, false);
-				if (!key)
-				{
-					success = false;
-					lua_pop(L, 2);
-					break;
-				}
-
-				Variant *value = fromLua(L, -1, false);
-				if (!value)
-				{
-					delete key;
-					success = false;
-					lua_pop(L, 2);
-					break;
-				}
-
-				table->push_back(std::make_pair(key, value));
-
+				table->emplace_back(fromLua(L, -2), fromLua(L, -1));
 				lua_pop(L, 1);
+
+				const auto &p = table->back();
+				if (p.first.getType() == UNKNOWN || p.second.getType() == UNKNOWN)
+				{
+					success = false;
+					break;
+				}
 			}
 
 			if (success)
-				v = new Variant(table);
+				return Variant(table);
 			else
-				delete_table(table);
+				delete table;
 		}
 		break;
 	}
+
+	Variant v;
+	v.type = UNKNOWN;
 	return v;
 }
 
-void Variant::toLua(lua_State *L)
+void Variant::toLua(lua_State *L) const
 {
 	switch (type)
 	{
 	case BOOLEAN:
 		lua_pushboolean(L, data.boolean);
 		break;
-	case CHARACTER:
-		lua_pushlstring(L, &data.character, 1);
-		break;
 	case NUMBER:
 		lua_pushnumber(L, data.number);
 		break;
 	case STRING:
-		lua_pushlstring(L, data.string.str, data.string.len);
+		lua_pushlstring(L, data.string->str, data.string->len);
+		break;
+	case SMALLSTRING:
+		lua_pushlstring(L, data.smallstring.str, data.smallstring.len);
 		break;
 	case LUSERDATA:
 		lua_pushlightuserdata(L, data.userdata);
 		break;
 	case FUSERDATA:
 		if (udatatype != INVALID_ID)
-		{
-			const char *name = NULL;
-			love::types.find(udatatype, name);
-			luax_pushtype(L, name, flags, (love::Object *) data.userdata);
-		}
+			luax_pushtype(L, udatatype, (love::Object *) data.userdata);
 		else
 			lua_pushlightuserdata(L, data.userdata);
 		// I know this is not the same
@@ -235,15 +250,22 @@ void Variant::toLua(lua_State *L)
 		// I can do (at the moment).
 		break;
 	case TABLE:
-		lua_newtable(L);
-		for (size_t i = 0; i < data.table->size(); ++i)
+	{
+		std::vector<std::pair<Variant, Variant>> *table = data.table->table;
+		int tsize = (int) table->size();
+
+		lua_createtable(L, 0, tsize);
+
+		for (int i = 0; i < tsize; ++i)
 		{
-			std::pair<Variant*, Variant*> &kv = data.table->at(i);
-			kv.first->toLua(L);
-			kv.second->toLua(L);
+			std::pair<Variant, Variant> &kv = (*table)[i];
+			kv.first.toLua(L);
+			kv.second.toLua(L);
 			lua_settable(L, -3);
 		}
+
 		break;
+	}
 	case NIL:
 	default:
 		lua_pushnil(L);
