@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2015 LOVE Development Team
+ * Copyright (c) 2006-2016 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -25,7 +25,7 @@
 namespace love
 {
 
-void delay(unsigned int ms)
+void sleep(unsigned int ms)
 {
 	SDL_Delay(ms);
 }
