@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2015 LOVE Development Team
+ * Copyright (c) 2006-2016 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -28,66 +28,56 @@ namespace love
 namespace event
 {
 
-Message::Message(const std::string &name, Variant *a, Variant *b, Variant *c, Variant *d)
+Message::Message(const std::string &name, const std::vector<Variant> &vargs)
 	: name(name)
-	, nargs(0)
+	, args(vargs)
 {
-	args[0] = a;
-	args[1] = b;
-	args[2] = c;
-	args[3] = d;
-	for (int i = 0; i < 4; i++)
-	{
-		if (!args[i])
-			break;
-		args[i]->retain();
-		nargs++;
-	}
 }
 
 Message::~Message()
 {
-	for (int i = 0; i < nargs; i++)
-		args[i]->release();
 }
 
 int Message::toLua(lua_State *L)
 {
 	luax_pushstring(L, name);
-	for (int i = 0; i < nargs; i++)
-		args[i]->toLua(L);
-	return nargs+1;
+
+	for (const Variant &v : args)
+		v.toLua(L);
+
+	return (int) args.size() + 1;
 }
 
 Message *Message::fromLua(lua_State *L, int n)
 {
 	std::string name = luax_checkstring(L, n);
+	std::vector<Variant> vargs;
+
+	int count = lua_gettop(L) - n;
 	n++;
-	Message *m = new Message(name);
-	for (int i = 0; i < 4; i++)
+
+	Variant varg;
+
+	for (int i = 0; i < count; i++)
 	{
 		if (lua_isnoneornil(L, n+i))
 			break;
-		m->args[i] = Variant::fromLua(L, n+i);
-		if (!m->args[i])
-		{
-			delete m;
-			luaL_error(L, "Argument %d can't be stored safely\nExpected boolean, number, string or userdata.", n+i);
-			return NULL;
-		}
-		m->nargs++;
-	}
-	return m;
-}
 
-Event::Event()
-{
-	mutex = thread::newMutex();
+		vargs.push_back(Variant::fromLua(L, n+i));
+
+		if (vargs.back().getType() == Variant::UNKNOWN)
+		{
+			vargs.clear();
+			luaL_error(L, "Argument %d can't be stored safely\nExpected boolean, number, string or userdata.", n+i);
+			return nullptr;
+		}
+	}
+
+	return new Message(name, vargs);
 }
 
 Event::~Event()
 {
-	delete mutex;
 }
 
 void Event::push(Message *msg)
