@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2015 LOVE Development Team
+ * Copyright (c) 2006-2016 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -26,6 +26,7 @@
 
 // C++
 #include <algorithm>
+#include <limits>
 
 namespace love
 {
@@ -62,8 +63,12 @@ namespace
 
 
 Shader *Shader::current = nullptr;
+Shader *Shader::defaultShader = nullptr;
+Shader *Shader::defaultVideoShader = nullptr;
+
+Shader::ShaderSource Shader::defaultCode[Graphics::RENDERER_MAX_ENUM][2];
+Shader::ShaderSource Shader::defaultVideoCode[Graphics::RENDERER_MAX_ENUM][2];
 
-GLint Shader::maxTexUnits = 0;
 std::vector<int> Shader::textureCounters;
 
 Shader::Shader(const ShaderSource &source)
@@ -73,20 +78,15 @@ Shader::Shader(const ShaderSource &source)
 	, builtinAttributes()
 	, lastCanvas((Canvas *) -1)
 	, lastViewport()
+	, lastPointSize(0.0f)
+	, videoTextureUnits()
 {
 	if (source.vertex.empty() && source.pixel.empty())
 		throw love::Exception("Cannot create shader: no source code!");
 
-	if (maxTexUnits <= 0)
-	{
-		GLint maxtexunits;
-		glGetIntegerv(GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS, &maxtexunits);
-		maxTexUnits = std::max(maxtexunits - 1, 0);
-	}
-
 	// initialize global texture id counters if needed
-	if (textureCounters.size() < (size_t) maxTexUnits)
-		textureCounters.resize(maxTexUnits, 0);
+	if ((int) textureCounters.size() < gl.getMaxTextureUnits() - 1)
+		textureCounters.resize(gl.getMaxTextureUnits() - 1, 0);
 
 	// load shader source and create program object
 	loadVolatile();
@@ -126,9 +126,6 @@ GLuint Shader::compileCode(ShaderStage stage, const std::string &code)
 		break;
 	}
 
-	// clear existing errors
-	while (glGetError() != GL_NO_ERROR);
-
 	GLuint shaderid = glCreateShader(glstage);
 
 	if (shaderid == 0)
@@ -181,6 +178,11 @@ void Shader::mapActiveUniforms()
 
 	uniforms.clear();
 
+	GLint activeprogram = 0;
+	glGetIntegerv(GL_CURRENT_PROGRAM, &activeprogram);
+
+	gl.useProgram(program);
+
 	GLint numuniforms;
 	glGetProgramiv(program, GL_ACTIVE_UNIFORMS, &numuniforms);
 
@@ -190,13 +192,21 @@ void Shader::mapActiveUniforms()
 	for (int i = 0; i < numuniforms; i++)
 	{
 		GLsizei namelen = 0;
-		Uniform u = {};
+		GLenum gltype = 0;
+		UniformInfo u = {};
 
-		glGetActiveUniform(program, (GLuint) i, bufsize, &namelen, &u.count, &u.type, cname);
+		glGetActiveUniform(program, (GLuint) i, bufsize, &namelen, &u.count, &gltype, cname);
 
 		u.name = std::string(cname, (size_t) namelen);
 		u.location = glGetUniformLocation(program, u.name.c_str());
-		u.baseType = getUniformBaseType(u.type);
+		u.baseType = getUniformBaseType(gltype);
+		u.components = getUniformTypeSize(gltype);
+
+		// Initialize all samplers to 0. Both GLSL and GLSL ES are supposed to
+		// do this themselves, but some Android devices (galaxy tab 3 and 4)
+		// don't seem to do it...
+		if (u.baseType == UNIFORM_SAMPLER)
+			glUniform1i(u.location, 0);
 
 		// glGetActiveUniform appends "[0]" to the end of array uniform names...
 		if (u.name.length() > 3)
@@ -214,27 +224,47 @@ void Shader::mapActiveUniforms()
 		if (u.location != -1)
 			uniforms[u.name] = u;
 	}
+
+	gl.useProgram(activeprogram);
 }
 
 bool Shader::loadVolatile()
 {
+	OpenGL::TempDebugGroup debuggroup("Shader load");
+
     // Recreating the shader program will invalidate uniforms that rely on these.
     lastCanvas = (Canvas *) -1;
     lastViewport = OpenGL::Viewport();
 
+	lastPointSize = -1.0f;
+
+	// Invalidate the cached matrices by setting some elements to NaN.
+	float nan = std::numeric_limits<float>::quiet_NaN();
+	lastProjectionMatrix.setTranslation(nan, nan);
+	lastTransformMatrix.setTranslation(nan, nan);
+
+	for (int i = 0; i < 3; i++)
+		videoTextureUnits[i] = 0;
+
 	// zero out active texture list
 	activeTexUnits.clear();
-	activeTexUnits.insert(activeTexUnits.begin(), maxTexUnits, 0);
+	activeTexUnits.insert(activeTexUnits.begin(), gl.getMaxTextureUnits() - 1, 0);
 
 	std::vector<GLuint> shaderids;
 
+	bool gammacorrect = graphics::isGammaCorrect();
+	const ShaderSource *defaults = &defaultCode[Graphics::RENDERER_OPENGL][gammacorrect ? 1 : 0];
+	if (GLAD_ES_VERSION_2_0)
+		defaults = &defaultCode[Graphics::RENDERER_OPENGLES][gammacorrect ? 1 : 0];
+
+	// The shader program must have both vertex and pixel shader stages.
+	const std::string &vertexcode = shaderSource.vertex.empty() ? defaults->vertex : shaderSource.vertex;
+	const std::string &pixelcode = shaderSource.pixel.empty() ? defaults->pixel : shaderSource.pixel;
+
 	try
 	{
-		if (!shaderSource.vertex.empty())
-			shaderids.push_back(compileCode(STAGE_VERTEX, shaderSource.vertex));
-
-		if (!shaderSource.pixel.empty())
-			shaderids.push_back(compileCode(STAGE_PIXEL, shaderSource.pixel));
+		shaderids.push_back(compileCode(STAGE_VERTEX, vertexcode));
+		shaderids.push_back(compileCode(STAGE_PIXEL, pixelcode));
 	}
 	catch (love::Exception &)
 	{
@@ -243,9 +273,6 @@ bool Shader::loadVolatile()
 		throw;
 	}
 
-	if (shaderids.empty())
-		throw love::Exception("Cannot create shader: no valid source code!");
-
 	program = glCreateProgram();
 
 	if (program == 0)
@@ -259,18 +286,10 @@ bool Shader::loadVolatile()
 		glAttachShader(program, id);
 
 	// Bind generic vertex attribute indices to names in the shader.
-	for (int i = 0; i < int(OpenGL::ATTRIB_MAX_ENUM); i++)
+	for (int i = 0; i < int(ATTRIB_MAX_ENUM); i++)
 	{
-		OpenGL::VertexAttrib attrib = (OpenGL::VertexAttrib) i;
-
-		// FIXME: We skip this both because pseudo-instancing is temporarily
-		// disabled (see graphics.lua), and because binding a non-existant
-		// attribute name to a location causes a shader linker warning.
-		if (attrib == OpenGL::ATTRIB_PSEUDO_INSTANCE_ID)
-			continue;
-
 		const char *name = nullptr;
-		if (attribNames.find(attrib, name))
+		if (attribNames.find((VertexAttribID) i, name))
 			glBindAttribLocation(program, i, (const GLchar *) name);
 	}
 
@@ -291,13 +310,13 @@ bool Shader::loadVolatile()
 		throw love::Exception("Cannot link shader program object:\n%s", warnings.c_str());
 	}
 
-	// Retreive all active uniform variables in this shader from OpenGL.
+	// Get all active uniform variables in this shader from OpenGL.
 	mapActiveUniforms();
 
-	for (int i = 0; i < int(OpenGL::ATTRIB_MAX_ENUM); i++)
+	for (int i = 0; i < int(ATTRIB_MAX_ENUM); i++)
 	{
 		const char *name = nullptr;
-		if (attribNames.find(OpenGL::VertexAttrib(i), name))
+		if (attribNames.find(VertexAttribID(i), name))
 			builtinAttributes[i] = glGetAttribLocation(program, name);
 		else
 			builtinAttributes[i] = -1;
@@ -308,7 +327,7 @@ bool Shader::loadVolatile()
 		// make sure glUseProgram gets called.
 		current = nullptr;
 		attach();
-        checkSetScreenParams();
+		checkSetBuiltinUniforms();
 	}
 
 	return true;
@@ -317,7 +336,7 @@ bool Shader::loadVolatile()
 void Shader::unloadVolatile()
 {
 	if (current == this)
-		glUseProgram(0);
+		gl.useProgram(0);
 
 	if (program != 0)
 	{
@@ -334,7 +353,9 @@ void Shader::unloadVolatile()
 
 	// active texture list is probably invalid, clear it
 	activeTexUnits.clear();
-	activeTexUnits.insert(activeTexUnits.begin(), maxTexUnits, 0);
+	activeTexUnits.resize(gl.getMaxTextureUnits() - 1, 0);
+
+	attributes.clear();
 
 	// same with uniform location list
 	uniforms.clear();
@@ -387,43 +408,430 @@ void Shader::attach(bool temporary)
 {
 	if (current != this)
 	{
-		glUseProgram(program);
+		gl.useProgram(program);
 		current = this;
 		// retain/release happens in Graphics::setShader.
-	}
 
-	if (!temporary)
-	{
-		// make sure all sent textures are properly bound to their respective texture units
-		// note: list potentially contains texture ids of deleted/invalid textures!
-		for (size_t i = 0; i < activeTexUnits.size(); ++i)
+		if (!temporary)
 		{
-			if (activeTexUnits[i] > 0)
-				gl.bindTextureToUnit(activeTexUnits[i], i + 1, false);
-		}
+			// make sure all sent textures are properly bound to their respective texture units
+			// note: list potentially contains texture ids of deleted/invalid textures!
+			for (size_t i = 0; i < activeTexUnits.size(); ++i)
+			{
+				if (activeTexUnits[i] > 0)
+					gl.bindTextureToUnit(activeTexUnits[i], i + 1, false);
+			}
 
-		// We always want to use texture unit 0 for everyhing else.
-		gl.setTextureUnit(0);
+			// We always want to use texture unit 0 for everyhing else.
+			gl.setTextureUnit(0);
+		}
 	}
 }
 
 void Shader::detach()
 {
+	if (defaultShader)
+	{
+		if (current != defaultShader)
+			defaultShader->attach();
+
+		return;
+	}
+
 	if (current != nullptr)
-		glUseProgram(0);
+		gl.useProgram(0);
 
 	current = nullptr;
 }
 
-const Shader::Uniform &Shader::getUniform(const std::string &name) const
+const Shader::UniformInfo *Shader::getUniformInfo(const std::string &name) const
 {
-	std::map<std::string, Uniform>::const_iterator it = uniforms.find(name);
+	const auto it = uniforms.find(name);
 
 	if (it == uniforms.end())
-		throw love::Exception("Variable '%s' does not exist.\n"
-		                      "A common error is to define but not use the variable.", name.c_str());
+		return nullptr;
 
-	return it->second;
+	return &(it->second);
+}
+
+void Shader::sendInts(const UniformInfo *info, const int *vec, int count)
+{
+	if (info->baseType != UNIFORM_INT && info->baseType != UNIFORM_BOOL)
+		return;
+
+	TemporaryAttacher attacher(this);
+
+	int location = info->location;
+
+	switch (info->components)
+	{
+	case 4:
+		glUniform4iv(location, count, vec);
+		break;
+	case 3:
+		glUniform3iv(location, count, vec);
+		break;
+	case 2:
+		glUniform2iv(location, count, vec);
+		break;
+	case 1:
+	default:
+		glUniform1iv(location, count, vec);
+		break;
+	}
+}
+
+void Shader::sendFloats(const UniformInfo *info, const float *vec, int count)
+{
+	if (info->baseType != UNIFORM_FLOAT && info->baseType != UNIFORM_BOOL)
+		return;
+
+	TemporaryAttacher attacher(this);
+
+	int location = info->location;
+
+	switch (info->components)
+	{
+	case 4:
+		glUniform4fv(location, count, vec);
+		break;
+	case 3:
+		glUniform3fv(location, count, vec);
+		break;
+	case 2:
+		glUniform2fv(location, count, vec);
+		break;
+	case 1:
+	default:
+		glUniform1fv(location, count, vec);
+		break;
+	}
+}
+
+void Shader::sendMatrices(const UniformInfo *info, const float *m, int count)
+{
+	if (info->baseType != UNIFORM_MATRIX)
+		return;
+
+	TemporaryAttacher attacher(this);
+
+	int location = info->location;
+
+	switch (info->components)
+	{
+	case 4:
+		glUniformMatrix4fv(location, count, GL_FALSE, m);
+		break;
+	case 3:
+		glUniformMatrix3fv(location, count, GL_FALSE, m);
+		break;
+	case 2:
+	default:
+		glUniformMatrix2fv(location, count, GL_FALSE, m);
+		break;
+	}
+}
+
+void Shader::sendTexture(const UniformInfo *info, Texture *texture)
+{
+	if (info->baseType != UNIFORM_SAMPLER)
+		return;
+
+	GLuint gltex = *(GLuint *) texture->getHandle();
+
+	TemporaryAttacher attacher(this);
+
+	int texunit = getTextureUnit(info->name);
+
+	// bind texture to assigned texture unit and send uniform to shader program
+	gl.bindTextureToUnit(gltex, texunit, true);
+
+	glUniform1i(info->location, texunit);
+
+	// increment global shader texture id counter for this texture unit, if we haven't already
+	if (activeTexUnits[texunit-1] == 0)
+		++textureCounters[texunit-1];
+
+	// store texture id so it can be re-bound to the proper texture unit later
+	activeTexUnits[texunit-1] = gltex;
+
+	retainObject(info->name, texture);
+}
+
+void Shader::retainObject(const std::string &name, Object *object)
+{
+	object->retain();
+
+	auto it = boundRetainables.find(name);
+	if (it != boundRetainables.end())
+		it->second->release();
+
+	boundRetainables[name] = object;
+}
+
+int Shader::getTextureUnit(const std::string &name)
+{
+	auto it = texUnitPool.find(name);
+
+	if (it != texUnitPool.end())
+		return it->second;
+
+	int texunit = 1;
+
+	// prefer texture units which are unused by all other shaders
+	auto freeunit_it = std::find(textureCounters.begin(), textureCounters.end(), 0);
+
+	if (freeunit_it != textureCounters.end())
+	{
+		// we don't want to use unit 0
+		texunit = (int) std::distance(textureCounters.begin(), freeunit_it) + 1;
+	}
+	else
+	{
+		// no completely unused texture units exist, try to use next free slot in our own list
+		auto nextunit_it = std::find(activeTexUnits.begin(), activeTexUnits.end(), 0);
+
+		if (nextunit_it == activeTexUnits.end())
+			throw love::Exception("No more texture units available for shader.");
+
+		// we don't want to use unit 0
+		texunit = (int) std::distance(activeTexUnits.begin(), nextunit_it) + 1;
+	}
+
+	texUnitPool[name] = texunit;
+	return texunit;
+}
+
+Shader::UniformType Shader::getExternVariable(const std::string &name, int &components, int &count)
+{
+	auto it = uniforms.find(name);
+
+	if (it == uniforms.end())
+	{
+		components = 0;
+		count = 0;
+		return UNIFORM_UNKNOWN;
+	}
+
+	components = it->second.components;
+	count = (int) it->second.count;
+
+	// Legacy support. This whole function is gone in 0.11 anyway.
+	if (it->second.baseType == UNIFORM_MATRIX)
+		return UNIFORM_FLOAT;
+	return it->second.baseType;
+}
+
+GLint Shader::getAttribLocation(const std::string &name)
+{
+	auto it = attributes.find(name);
+	if (it != attributes.end())
+		return it->second;
+
+	GLint location = glGetAttribLocation(program, name.c_str());
+
+	attributes[name] = location;
+	return location;
+}
+
+bool Shader::hasVertexAttrib(VertexAttribID attrib) const
+{
+	return builtinAttributes[int(attrib)] != -1;
+}
+
+void Shader::setVideoTextures(GLuint ytexture, GLuint cbtexture, GLuint crtexture)
+{
+	TemporaryAttacher attacher(this);
+
+	// Set up the texture units that will be used by the shader to sample from
+	// the textures, if they haven't been set up yet.
+	if (videoTextureUnits[0] == 0)
+	{
+		const GLint locs[3] = {
+			builtinUniforms[BUILTIN_VIDEO_Y_CHANNEL],
+			builtinUniforms[BUILTIN_VIDEO_CB_CHANNEL],
+			builtinUniforms[BUILTIN_VIDEO_CR_CHANNEL]
+		};
+
+		const char *names[3] = {nullptr, nullptr, nullptr};
+		builtinNames.find(BUILTIN_VIDEO_Y_CHANNEL,  names[0]);
+		builtinNames.find(BUILTIN_VIDEO_CB_CHANNEL, names[1]);
+		builtinNames.find(BUILTIN_VIDEO_CR_CHANNEL, names[2]);
+
+		for (int i = 0; i < 3; i++)
+		{
+			if (locs[i] >= 0 && names[i] != nullptr)
+			{
+				videoTextureUnits[i] = getTextureUnit(names[i]);
+
+				// Increment global shader texture id counter for this texture
+				// unit, if we haven't already.
+				if (activeTexUnits[videoTextureUnits[i] - 1] == 0)
+					++textureCounters[videoTextureUnits[i] - 1];
+
+				glUniform1i(locs[i], videoTextureUnits[i]);
+			}
+		}
+	}
+
+	const GLuint textures[3] = {ytexture, cbtexture, crtexture};
+
+	// Bind the textures to their respective texture units.
+	for (int i = 0; i < 3; i++)
+	{
+		if (videoTextureUnits[i] != 0)
+		{
+			// Store texture id so it can be re-bound later.
+			activeTexUnits[videoTextureUnits[i] - 1] = textures[i];
+			gl.bindTextureToUnit(textures[i], videoTextureUnits[i], false);
+		}
+	}
+
+	gl.setTextureUnit(0);
+}
+
+void Shader::checkSetScreenParams()
+{
+	OpenGL::Viewport view = gl.getViewport();
+
+	if (view == lastViewport && lastCanvas == Canvas::current)
+		return;
+
+	// In the shader, we do pixcoord.y = gl_FragCoord.y * params.z + params.w.
+	// This lets us flip pixcoord.y when needed, to be consistent (drawing with
+	// no Canvas active makes the y-values for pixel coordinates flipped.)
+	GLfloat params[] = {
+		(GLfloat) view.w, (GLfloat) view.h,
+		0.0f, 0.0f,
+	};
+
+	if (Canvas::current != nullptr)
+	{
+		// No flipping: pixcoord.y = gl_FragCoord.y * 1.0 + 0.0.
+		params[2] = 1.0f;
+		params[3] = 0.0f;
+	}
+	else
+	{
+		// gl_FragCoord.y is flipped when drawing to the screen, so we un-flip:
+		// pixcoord.y = gl_FragCoord.y * -1.0 + height.
+		params[2] = -1.0f;
+		params[3] = (GLfloat) view.h;
+	}
+
+	GLint location = builtinUniforms[BUILTIN_SCREEN_SIZE];
+
+	if (location >= 0)
+	{
+		TemporaryAttacher attacher(this);
+		glUniform4fv(location, 1, params);
+	}
+
+	lastCanvas = Canvas::current;
+	lastViewport = view;
+}
+
+void Shader::checkSetPointSize(float size)
+{
+	if (size == lastPointSize)
+		return;
+
+	GLint location = builtinUniforms[BUILTIN_POINT_SIZE];
+
+	if (location >= 0)
+	{
+		TemporaryAttacher attacher(this);
+		glUniform1f(location, size);
+	}
+
+	lastPointSize = size;
+}
+
+void Shader::checkSetBuiltinUniforms()
+{
+	checkSetScreenParams();
+
+	// We use a more efficient method for sending transformation matrices to
+	// the GPU on desktop GL.
+	if (GLAD_ES_VERSION_2_0)
+	{
+		checkSetPointSize(gl.getPointSize());
+
+		const Matrix4 &curxform = gl.matrices.transform.back();
+		const Matrix4 &curproj = gl.matrices.projection.back();
+
+		TemporaryAttacher attacher(this);
+
+		bool tpmatrixneedsupdate = false;
+
+		// Only upload the matrices if they've changed.
+		if (memcmp(curxform.getElements(), lastTransformMatrix.getElements(), sizeof(float) * 16) != 0)
+		{
+			GLint location = builtinUniforms[BUILTIN_TRANSFORM_MATRIX];
+			if (location >= 0)
+				glUniformMatrix4fv(location, 1, GL_FALSE, curxform.getElements());
+
+			// Also upload the re-calculated normal matrix, if possible. The
+			// normal matrix is the transpose of the inverse of the rotation
+			// portion (top-left 3x3) of the transform matrix.
+			location = builtinUniforms[BUILTIN_NORMAL_MATRIX];
+			if (location >= 0)
+			{
+				Matrix3 normalmatrix = Matrix3(curxform).transposedInverse();
+				glUniformMatrix3fv(location, 1, GL_FALSE, normalmatrix.getElements());
+			}
+
+			tpmatrixneedsupdate = true;
+			lastTransformMatrix = curxform;
+		}
+
+		if (memcmp(curproj.getElements(), lastProjectionMatrix.getElements(), sizeof(float) * 16) != 0)
+		{
+			GLint location = builtinUniforms[BUILTIN_PROJECTION_MATRIX];
+			if (location >= 0)
+				glUniformMatrix4fv(location, 1, GL_FALSE, curproj.getElements());
+
+			tpmatrixneedsupdate = true;
+			lastProjectionMatrix = curproj;
+		}
+
+		if (tpmatrixneedsupdate)
+		{
+			GLint location = builtinUniforms[BUILTIN_TRANSFORM_PROJECTION_MATRIX];
+			if (location >= 0)
+			{
+				Matrix4 tp_matrix(curproj * curxform);
+				glUniformMatrix4fv(location, 1, GL_FALSE, tp_matrix.getElements());
+			}
+		}
+	}
+}
+
+const std::map<std::string, Object *> &Shader::getBoundRetainables() const
+{
+	return boundRetainables;
+}
+
+std::string Shader::getGLSLVersion()
+{
+	const char *tmp = (const char *) glGetString(GL_SHADING_LANGUAGE_VERSION);
+
+	if (tmp == nullptr)
+		return "0.0";
+
+	// the version string always begins with a version number of the format
+	//   major_number.minor_number
+	// or
+	//   major_number.minor_number.release_number
+	// we can keep release_number, since it does not affect the check below.
+	std::string versionstring(tmp);
+	size_t minorendpos = versionstring.find(' ');
+	return versionstring.substr(0, minorendpos);
+}
+
+bool Shader::isSupported()
+{
+	return GLAD_ES_VERSION_2_0 || (getGLSLVersion() >= "1.2");
 }
 
 int Shader::getUniformTypeSize(GLenum type) const
@@ -453,10 +861,8 @@ int Shader::getUniformTypeSize(GLenum type) const
 	case GL_BOOL_VEC4:
 		return 4;
 	default:
-		break;
+		return 1;
 	}
-
-	return 1;
 }
 
 Shader::UniformType Shader::getUniformBaseType(GLenum type) const
@@ -472,317 +878,43 @@ Shader::UniformType Shader::getUniformBaseType(GLenum type) const
 	case GL_FLOAT_VEC2:
 	case GL_FLOAT_VEC3:
 	case GL_FLOAT_VEC4:
+		return UNIFORM_FLOAT;
 	case GL_FLOAT_MAT2:
 	case GL_FLOAT_MAT3:
 	case GL_FLOAT_MAT4:
-		return UNIFORM_FLOAT;
+	case GL_FLOAT_MAT2x3:
+	case GL_FLOAT_MAT2x4:
+	case GL_FLOAT_MAT3x2:
+	case GL_FLOAT_MAT3x4:
+	case GL_FLOAT_MAT4x2:
+	case GL_FLOAT_MAT4x3:
+		return UNIFORM_MATRIX;
 	case GL_BOOL:
 	case GL_BOOL_VEC2:
 	case GL_BOOL_VEC3:
 	case GL_BOOL_VEC4:
 		return UNIFORM_BOOL;
 	case GL_SAMPLER_1D:
+	case GL_SAMPLER_1D_SHADOW:
+	case GL_SAMPLER_1D_ARRAY:
+	case GL_SAMPLER_1D_ARRAY_SHADOW:
 	case GL_SAMPLER_2D:
+	case GL_SAMPLER_2D_MULTISAMPLE:
+	case GL_SAMPLER_2D_MULTISAMPLE_ARRAY:
+	case GL_SAMPLER_2D_RECT:
+	case GL_SAMPLER_2D_RECT_SHADOW:
+	case GL_SAMPLER_2D_SHADOW:
+	case GL_SAMPLER_2D_ARRAY:
+	case GL_SAMPLER_2D_ARRAY_SHADOW:
 	case GL_SAMPLER_3D:
+	case GL_SAMPLER_CUBE:
+	case GL_SAMPLER_CUBE_SHADOW:
+	case GL_SAMPLER_CUBE_MAP_ARRAY:
+	case GL_SAMPLER_CUBE_MAP_ARRAY_SHADOW:
 		return UNIFORM_SAMPLER;
 	default:
-		break;
-	}
-
-	return UNIFORM_UNKNOWN;
-}
-
-void Shader::checkSetUniformError(const Uniform &u, int size, int count, UniformType sendtype) const
-{
-	if (!program)
-		throw love::Exception("No active shader program.");
-
-	int realsize = getUniformTypeSize(u.type);
-
-	if (size != realsize)
-		throw love::Exception("Value size of %d does not match variable size of %d.", size, realsize);
-
-	if ((u.count == 1 && count > 1) || count < 0)
-		throw love::Exception("Invalid number of values (expected %d, got %d).", u.count, count);
-
-	if (u.baseType == UNIFORM_SAMPLER && sendtype != u.baseType)
-		throw love::Exception("Cannot send a value of this type to an Image variable.");
-
-	if ((sendtype == UNIFORM_FLOAT && u.baseType == UNIFORM_INT) || (sendtype == UNIFORM_INT && u.baseType == UNIFORM_FLOAT))
-		throw love::Exception("Cannot convert between float and int.");
-}
-
-void Shader::sendInt(const std::string &name, int size, const GLint *vec, int count)
-{
-	TemporaryAttacher attacher(this);
-
-	const Uniform &u = getUniform(name);
-	checkSetUniformError(u, size, count, UNIFORM_INT);
-
-	switch (size)
-	{
-	case 4:
-		glUniform4iv(u.location, count, vec);
-		break;
-	case 3:
-		glUniform3iv(u.location, count, vec);
-		break;
-	case 2:
-		glUniform2iv(u.location, count, vec);
-		break;
-	case 1:
-	default:
-		glUniform1iv(u.location, count, vec);
-		break;
-	}
-}
-
-void Shader::sendFloat(const std::string &name, int size, const GLfloat *vec, int count)
-{
-	TemporaryAttacher attacher(this);
-
-	const Uniform &u = getUniform(name);
-	checkSetUniformError(u, size, count, UNIFORM_FLOAT);
-
-	switch (size)
-	{
-	case 4:
-		glUniform4fv(u.location, count, vec);
-		break;
-	case 3:
-		glUniform3fv(u.location, count, vec);
-		break;
-	case 2:
-		glUniform2fv(u.location, count, vec);
-		break;
-	case 1:
-	default:
-		glUniform1fv(u.location, count, vec);
-		break;
-	}
-}
-
-void Shader::sendMatrix(const std::string &name, int size, const GLfloat *m, int count)
-{
-	TemporaryAttacher attacher(this);
-
-	if (size < 2 || size > 4)
-	{
-		throw love::Exception("Invalid matrix size: %dx%d "
-							  "(can only set 2x2, 3x3 or 4x4 matrices.)", size,size);
-	}
-
-	const Uniform &u = getUniform(name);
-	checkSetUniformError(u, size, count, UNIFORM_FLOAT);
-
-	switch (size)
-	{
-	case 4:
-		glUniformMatrix4fv(u.location, count, GL_FALSE, m);
-		break;
-	case 3:
-		glUniformMatrix3fv(u.location, count, GL_FALSE, m);
-		break;
-	case 2:
-	default:
-		glUniformMatrix2fv(u.location, count, GL_FALSE, m);
-		break;
-	}
-}
-
-void Shader::sendTexture(const std::string &name, Texture *texture)
-{
-	GLuint gltex = texture->getGLTexture();
-
-	TemporaryAttacher attacher(this);
-
-	int texunit = getTextureUnit(name);
-
-	const Uniform &u = getUniform(name);
-	checkSetUniformError(u, 1, 1, UNIFORM_SAMPLER);
-
-	// bind texture to assigned texture unit and send uniform to shader program
-	gl.bindTextureToUnit(gltex, texunit, false);
-
-	glUniform1i(u.location, texunit);
-
-	// reset texture unit
-	gl.setTextureUnit(0);
-
-	// increment global shader texture id counter for this texture unit, if we haven't already
-	if (activeTexUnits[texunit-1] == 0)
-		++textureCounters[texunit-1];
-
-	// store texture id so it can be re-bound to the proper texture unit later
-	activeTexUnits[texunit-1] = gltex;
-
-	retainObject(name, texture);
-}
-
-void Shader::retainObject(const std::string &name, Object *object)
-{
-	auto it = boundRetainables.find(name);
-	if (it != boundRetainables.end())
-		it->second->release();
-
-	object->retain();
-	boundRetainables[name] = object;
-}
-
-int Shader::getTextureUnit(const std::string &name)
-{
-	auto it = texUnitPool.find(name);
-
-	if (it != texUnitPool.end())
-		return it->second;
-
-	int texunit = 1;
-
-	// prefer texture units which are unused by all other shaders
-	auto freeunit_it = std::find(textureCounters.begin(), textureCounters.end(), 0);
-
-	if (freeunit_it != textureCounters.end())
-	{
-		// we don't want to use unit 0
-		texunit = std::distance(textureCounters.begin(), freeunit_it) + 1;
-	}
-	else
-	{
-		// no completely unused texture units exist, try to use next free slot in our own list
-		auto nextunit_it = std::find(activeTexUnits.begin(), activeTexUnits.end(), 0);
-
-		if (nextunit_it == activeTexUnits.end())
-			throw love::Exception("No more texture units available for shader.");
-
-		// we don't want to use unit 0
-		texunit = std::distance(activeTexUnits.begin(), nextunit_it) + 1;
-	}
-
-	texUnitPool[name] = texunit;
-	return texunit;
-}
-
-Shader::UniformType Shader::getExternVariable(const std::string &name, int &components, int &count)
-{
-	auto it = uniforms.find(name);
-
-	if (it == uniforms.end())
-	{
-		components = 0;
-		count = 0;
 		return UNIFORM_UNKNOWN;
 	}
-
-	components = getUniformTypeSize(it->second.type);
-	count = (int) it->second.count;
-
-	return it->second.baseType;
-}
-
-bool Shader::hasVertexAttrib(OpenGL::VertexAttrib attrib) const
-{
-	return builtinAttributes[int(attrib)] != -1;
-}
-
-bool Shader::hasBuiltinUniform(BuiltinUniform builtin) const
-{
-	return builtinUniforms[int(builtin)] != -1;
-}
-
-bool Shader::sendBuiltinFloat(BuiltinUniform builtin, int size, const GLfloat *vec, int count)
-{
-	if (!hasBuiltinUniform(builtin))
-		return false;
-
-	GLint location = builtinUniforms[int(builtin)];
-
-	TemporaryAttacher attacher(this);
-
-	switch (size)
-	{
-	case 1:
-		glUniform1fv(location, count, vec);
-		break;
-	case 2:
-		glUniform2fv(location, count, vec);
-		break;
-	case 3:
-		glUniform3fv(location, count, vec);
-		break;
-	case 4:
-		glUniform4fv(location, count, vec);
-		break;
-	default:
-		return false;
-	}
-	
-	return true;
-}
-
-void Shader::checkSetScreenParams()
-{
-	OpenGL::Viewport view = gl.getViewport();
-
-	if (view == lastViewport && lastCanvas == Canvas::current)
-		return;
-
-	// In the shader, we do pixcoord.y = gl_FragCoord.y * params.z + params.w.
-	// This lets us flip pixcoord.y when needed, to be consistent (Canvases
-	// have flipped y-values for pixel coordinates.)
-	GLfloat params[] = {
-		(GLfloat) view.w, (GLfloat) view.h,
-		0.0f, 0.0f,
-	};
-
-	if (Canvas::current != nullptr)
-	{
-		// gl_FragCoord.y is flipped in Canvases, so we un-flip:
-		// pixcoord.y = gl_FragCoord.y * -1.0 + height.
-		params[2] = -1.0f;
-		params[3] = (GLfloat) view.h;
-	}
-	else
-	{
-		// No flipping: pixcoord.y = gl_FragCoord.y * 1.0 + 0.0.
-		params[2] = 1.0f;
-		params[3] = 0.0f;
-	}
-
-	sendBuiltinFloat(BUILTIN_SCREEN_SIZE, 4, params, 1);
-
-	lastCanvas = Canvas::current;
-	lastViewport = view;
-}
-
-const std::map<std::string, Object *> &Shader::getBoundRetainables() const
-{
-	return boundRetainables;
-}
-
-std::string Shader::getGLSLVersion()
-{
-	const char *tmp = nullptr;
-
-	// GL_SHADING_LANGUAGE_VERSION isn't available in OpenGL < 2.0.
-	if (GLEE_VERSION_2_0 || GLEE_ARB_shading_language_100)
-		tmp = (const char *) glGetString(GL_SHADING_LANGUAGE_VERSION);
-
-	if (tmp == nullptr)
-		return "0.0";
-
-	// the version string always begins with a version number of the format
-	//   major_number.minor_number
-	// or
-	//   major_number.minor_number.release_number
-	// we can keep release_number, since it does not affect the check below.
-	std::string versionstring(tmp);
-	size_t minorendpos = versionstring.find(' ');
-	return versionstring.substr(0, minorendpos);
-}
-
-bool Shader::isSupported()
-{
-	return GLEE_VERSION_2_0 && getGLSLVersion() >= "1.2";
 }
 
 bool Shader::getConstant(const char *in, UniformType &out)
@@ -795,6 +927,16 @@ bool Shader::getConstant(UniformType in, const char *&out)
 	return uniformTypes.find(in, out);
 }
 
+bool Shader::getConstant(const char *in, VertexAttribID &out)
+{
+	return attribNames.find(in, out);
+}
+
+bool Shader::getConstant(VertexAttribID in, const char *&out)
+{
+	return attribNames.find(in, out);
+}
+
 StringMap<Shader::ShaderStage, Shader::STAGE_MAX_ENUM>::Entry Shader::stageNameEntries[] =
 {
 	{"vertex", Shader::STAGE_VERTEX},
@@ -806,6 +948,7 @@ StringMap<Shader::ShaderStage, Shader::STAGE_MAX_ENUM> Shader::stageNames(Shader
 StringMap<Shader::UniformType, Shader::UNIFORM_MAX_ENUM>::Entry Shader::uniformTypeEntries[] =
 {
 	{"float", Shader::UNIFORM_FLOAT},
+	{"matrix", Shader::UNIFORM_MATRIX},
 	{"int", Shader::UNIFORM_INT},
 	{"bool", Shader::UNIFORM_BOOL},
 	{"image", Shader::UNIFORM_SAMPLER},
@@ -814,16 +957,27 @@ StringMap<Shader::UniformType, Shader::UNIFORM_MAX_ENUM>::Entry Shader::uniformT
 
 StringMap<Shader::UniformType, Shader::UNIFORM_MAX_ENUM> Shader::uniformTypes(Shader::uniformTypeEntries, sizeof(Shader::uniformTypeEntries));
 
-StringMap<OpenGL::VertexAttrib, OpenGL::ATTRIB_MAX_ENUM>::Entry Shader::attribNameEntries[] =
+StringMap<VertexAttribID, ATTRIB_MAX_ENUM>::Entry Shader::attribNameEntries[] =
 {
-	{"love_PseudoInstanceID", OpenGL::ATTRIB_PSEUDO_INSTANCE_ID},
+	{"VertexPosition", ATTRIB_POS},
+	{"VertexTexCoord", ATTRIB_TEXCOORD},
+	{"VertexColor", ATTRIB_COLOR},
+	{"ConstantColor", ATTRIB_CONSTANTCOLOR},
 };
 
-StringMap<OpenGL::VertexAttrib, OpenGL::ATTRIB_MAX_ENUM> Shader::attribNames(Shader::attribNameEntries, sizeof(Shader::attribNameEntries));
+StringMap<VertexAttribID, ATTRIB_MAX_ENUM> Shader::attribNames(Shader::attribNameEntries, sizeof(Shader::attribNameEntries));
 
 StringMap<Shader::BuiltinUniform, Shader::BUILTIN_MAX_ENUM>::Entry Shader::builtinNameEntries[] =
 {
+	{"TransformMatrix", Shader::BUILTIN_TRANSFORM_MATRIX},
+	{"ProjectionMatrix", Shader::BUILTIN_PROJECTION_MATRIX},
+	{"TransformProjectionMatrix", Shader::BUILTIN_TRANSFORM_PROJECTION_MATRIX},
+	{"NormalMatrix", Shader::BUILTIN_NORMAL_MATRIX},
+	{"love_PointSize", Shader::BUILTIN_POINT_SIZE},
 	{"love_ScreenSize", Shader::BUILTIN_SCREEN_SIZE},
+	{"love_VideoYChannel", Shader::BUILTIN_VIDEO_Y_CHANNEL},
+	{"love_VideoCbChannel", Shader::BUILTIN_VIDEO_CB_CHANNEL},
+	{"love_VideoCrChannel", Shader::BUILTIN_VIDEO_CR_CHANNEL},
 };
 
 StringMap<Shader::BuiltinUniform, Shader::BUILTIN_MAX_ENUM> Shader::builtinNames(Shader::builtinNameEntries, sizeof(Shader::builtinNameEntries));
