@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2015 LOVE Development Team
+ * Copyright (c) 2006-2016 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -19,7 +19,12 @@
  **/
 
 // LOVE
+#include "common/config.h"
 #include "wrap_Font.h"
+#include "wrap_Text.h"
+
+// C++
+#include <algorithm>
 
 namespace love
 {
@@ -30,7 +35,7 @@ namespace opengl
 
 Font *luax_checkfont(lua_State *L, int idx)
 {
-	return luax_checktype<Font>(L, idx, "Font", GRAPHICS_FONT_T);
+	return luax_checktype<Font>(L, idx, GRAPHICS_FONT_ID);
 }
 
 int w_Font_getHeight(lua_State *L)
@@ -52,17 +57,29 @@ int w_Font_getWidth(lua_State *L)
 int w_Font_getWrap(lua_State *L)
 {
 	Font *t = luax_checkfont(L, 1);
-	const char *str = luaL_checkstring(L, 2);
-	float wrap = (float) luaL_checknumber(L, 3);
-	int max_width = 0, numlines = 0;
 
-	luax_catchexcept(L, [&]() {
-		std::vector<std::string> lines = t->getWrap(str, wrap, &max_width);
-		numlines = lines.size();
-	});
+	std::vector<Font::ColoredString> text;
+	luax_checkcoloredstring(L, 2, text);
+
+	float wrap = (float) luaL_checknumber(L, 3);
+	int max_width = 0;
+	std::vector<std::string> lines;
+	std::vector<int> widths;
+
+	luax_catchexcept(L, [&]() { t->getWrap(text, wrap, lines, &widths); });
+
+	for (int width : widths)
+		max_width = std::max(max_width, width);
 
 	lua_pushinteger(L, max_width);
-	lua_pushinteger(L, numlines);
+	lua_createtable(L, (int) lines.size(), 0);
+
+	for (int i = 0; i < (int) lines.size(); i++)
+	{
+		lua_pushstring(L, lines[i].c_str());
+		lua_rawseti(L, -2, i + 1);
+	}
+
 	return 2;
 }
 
@@ -140,8 +157,7 @@ int w_Font_hasGlyphs(lua_State *L)
 	Font *t = luax_checkfont(L, 1);
 	bool hasglyph = false;
 
-	int count = lua_gettop(L) - 1;
-	count = count < 1 ? 1 : count;
+	int count = std::max(lua_gettop(L) - 1, 1);
 
 	luax_catchexcept(L, [&]() {
 		 for (int i = 2; i < count + 2; i++)
@@ -160,7 +176,19 @@ int w_Font_hasGlyphs(lua_State *L)
 	return 1;
 }
 
-static const luaL_Reg functions[] =
+int w_Font_setFallbacks(lua_State *L)
+{
+	Font *t = luax_checkfont(L, 1);
+	std::vector<Font *> fallbacks;
+
+	for (int i = 2; i <= lua_gettop(L); i++)
+		fallbacks.push_back(luax_checkfont(L, i));
+
+	luax_catchexcept(L, [&](){ t->setFallbacks(fallbacks); });
+	return 0;
+}
+
+static const luaL_Reg w_Font_functions[] =
 {
 	{ "getHeight", w_Font_getHeight },
 	{ "getWidth", w_Font_getWidth },
@@ -173,12 +201,13 @@ static const luaL_Reg functions[] =
 	{ "getDescent", w_Font_getDescent },
 	{ "getBaseline", w_Font_getBaseline },
 	{ "hasGlyphs", w_Font_hasGlyphs },
+	{ "setFallbacks", w_Font_setFallbacks },
 	{ 0, 0 }
 };
 
 extern "C" int luaopen_font(lua_State *L)
 {
-	return luax_register_type(L, "Font", functions);
+	return luax_register_type(L, GRAPHICS_FONT_ID, "Font", w_Font_functions, nullptr);
 }
 
 } // opengl
