@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2015 LOVE Development Team
+ * Copyright (c) 2006-2016 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -33,17 +33,6 @@ namespace opengl
 {
 
 Font *luax_checkfont(lua_State *L, int idx);
-int w_Font_getHeight(lua_State *L);
-int w_Font_getWidth(lua_State *L);
-int w_Font_getWrap(lua_State *L);
-int w_Font_setLineHeight(lua_State *L);
-int w_Font_getLineHeight(lua_State *L);
-int w_Font_setFilter(lua_State *L);
-int w_Font_getFilter(lua_State *L);
-int w_Font_getAscent(lua_State *L);
-int w_Font_getDescent(lua_State *L);
-int w_Font_getBaseline(lua_State *L);
-int w_Font_hasGlyphs(lua_State *L);
 extern "C" int luaopen_font(lua_State *L);
 
 } // opengl
