@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2015 LOVE Development Team
+ * Copyright (c) 2006-2016 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -22,6 +22,7 @@
 #define LOVE_GRAPHICS_OPENGL_WRAP_PROGRAM_H
 
 #include "common/runtime.h"
+#include "common/config.h"
 #include "Shader.h"
 
 namespace love
@@ -32,13 +33,6 @@ namespace opengl
 {
 
 Shader *luax_checkshader(lua_State *L, int idx);
-int w_Shader_getWarnings(lua_State *L);
-int w_Shader_sendInt(lua_State *L);
-int w_Shader_sendFloat(lua_State *L);
-int w_Shader_sendMatrix(lua_State *L);
-int w_Shader_sendTexture(lua_State *L);
-int w_Shader_send(lua_State *L);
-int w_Shader_getExternVariable(lua_State *L);
 extern "C" int luaopen_shader(lua_State *L);
 
 } // opengl
