@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2015 LOVE Development Team
+ * Copyright (c) 2006-2016 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -30,12 +30,6 @@ namespace love
 namespace system
 {
 
-int w_getOS(lua_State *L);
-int w_getProcessorCount(lua_State *L);
-int w_setClipboardText(lua_State *L);
-int w_getClipboardText(lua_State *L);
-int w_getPowerInfo(lua_State *L);
-int w_openURL(lua_State *L);
 extern "C" LOVE_EXPORT int luaopen_love_system(lua_State *L);
 
 } // system
