@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2015 LOVE Development Team
+ * Copyright (c) 2006-2016 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -29,9 +29,6 @@ namespace love
 {
 namespace thread
 {
-int w_newThread(lua_State *L);
-int w_newChannel(lua_State *L);
-int w_getChannel(lua_State *L);
 
 extern "C" LOVE_EXPORT int luaopen_love_thread(lua_State * L);
 
